@@ -4,6 +4,7 @@
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2010       Michiel D. Nauta
 # Copyright (C) 2011       Tim G L Lyons
+# Copyright (C) 2017       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,46 +25,50 @@
 Attribute class for Gramps.
 """
 
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 #
-# GRAMPS modules
+# Gramps modules
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 from .secondaryobj import SecondaryObject
 from .privacybase import PrivacyBase
 from .citationbase import CitationBase
 from .notebase import NoteBase
 from .attrtype import AttributeType
 from .const import IDENTICAL, EQUAL, DIFFERENT
+from ..const import GRAMPS_LOCALE as glocale
 
-#-------------------------------------------------------------------------
+_ = glocale.translation.gettext
+
+
+# -------------------------------------------------------------------------
 #
-# Root object for Attribute 
+# Root object for Attribute
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 class AttributeRoot(SecondaryObject, PrivacyBase):
     """
-    Provide a simple key/value pair for describing properties. 
+    Provide a simple key/value pair for describing properties.
     Used to store descriptive information.
-    
+
     In GEDCOM only used for Persons:
-    Individual attributes should describe situations that may be permanent or 
+    Individual attributes should describe situations that may be permanent or
     temporary (start at some date, end at some date, etc.), may be associated
-    to a place (a position held, residence, etc.) or may not (eye colour, 
-    height, caste, profession, etc.).  They may have sources and notes and 
-    media. 
+    to a place (a position held, residence, etc.) or may not (eye colour,
+    height, caste, profession, etc.).  They may have sources and notes and
+    media.
     Compare with :class:`~.event.Event`
-    
+
     Gramps at the moment does not support this GEDCOM Attribute structure.
     """
-    
+
     def __init__(self, source=None):
         """
         Create a new Attribute object, copying from the source if provided.
         """
         PrivacyBase.__init__(self, source)
-        
-        #type structure depends on inheriting classes
+
+        # type structure depends on inheriting classes
         self.type = None
         self.value = None
 
@@ -74,45 +79,7 @@ class AttributeRoot(SecondaryObject, PrivacyBase):
         """
         Convert the object to a serialized tuple of data.
         """
-        return (PrivacyBase.serialize(self),
-                self.type.serialize(), self.value)
-
-    def to_struct(self):
-        """
-        Convert the data held in this object to a structure (eg,
-        struct) that represents all the data elements.
-        
-        This method is used to recursively convert the object into a
-        self-documenting form that can easily be used for various
-        purposes, including diffs and queries.
-
-        These structures may be primitive Python types (string,
-        integer, boolean, etc.) or complex Python types (lists,
-        tuples, or dicts). If the return type is a dict, then the keys
-        of the dict match the fieldname of the object. If the return
-        struct (or value of a dict key) is a list, then it is a list
-        of structs. Otherwise, the struct is just the value of the
-        attribute.
-
-        :returns: Returns a struct containing the data of the object.
-        :rtype: dict
-        """
-        return {"_class": self.__class__.__name__,
-                "private": PrivacyBase.serialize(self),
-                "type": self.type.to_struct(), 
-                "value": self.value}
-
-    @classmethod
-    def from_struct(cls, struct):
-        """
-        Given a struct data representation, return a serialized object.
-
-        :returns: Returns a serialized object
-        """
-        default = Attribute()
-        return (PrivacyBase.from_struct(struct.get("private", default.private)),
-                AttributeType.from_struct(struct.get("type", {})), 
-                struct.get("value", default.value))
+        return (PrivacyBase.serialize(self), self.type.serialize(), self.value)
 
     def unserialize(self, data):
         """
@@ -145,7 +112,7 @@ class AttributeRoot(SecondaryObject, PrivacyBase):
         """
         Return the list of child secondary objects that may refer notes.
 
-        :returns: Returns the list of child secondary child objects that may 
+        :returns: Returns the list of child secondary child objects that may
                 refer notes.
         :rtype: list
         """
@@ -155,7 +122,7 @@ class AttributeRoot(SecondaryObject, PrivacyBase):
         """
         Return the list of child objects which may, directly or through
         their children, reference primary objects.
-        
+
         :returns: Returns the list of objects referencing primary objects.
         :rtype: list
         """
@@ -165,7 +132,7 @@ class AttributeRoot(SecondaryObject, PrivacyBase):
         """
         Return the list of (classname, handle) tuples for all directly
         referenced primary objects.
-        
+
         :returns: List of (classname, handle) tuples for referenced objects.
         :rtype: list
         """
@@ -216,13 +183,13 @@ class AttributeRoot(SecondaryObject, PrivacyBase):
         """Return the value of the Attribute instance."""
         return self.value
 
-#-------------------------------------------------------------------------
+
+# -------------------------------------------------------------------------
 #
-# Attribute for Person/Family/MediaObject/MediaRef
+# Attribute for Person/Family/Media/MediaRef
 #
-#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 class Attribute(AttributeRoot, CitationBase, NoteBase):
-        
     def __init__(self, source=None):
         """
         Create a new Attribute object, copying from the source if provided.
@@ -230,61 +197,25 @@ class Attribute(AttributeRoot, CitationBase, NoteBase):
         AttributeRoot.__init__(self, source)
         CitationBase.__init__(self, source)
         NoteBase.__init__(self, source)
-        
+
         if source:
             self.type = AttributeType(source.type)
             self.value = source.value
         else:
             self.type = AttributeType()
             self.value = ""
+
     def serialize(self):
         """
         Convert the object to a serialized tuple of data.
         """
-        return (PrivacyBase.serialize(self),
-                CitationBase.serialize(self),
-                NoteBase.serialize(self),
-                self.type.serialize(), self.value)
-
-    def to_struct(self):
-        """
-        Convert the data held in this object to a structure (eg,
-        struct) that represents all the data elements.
-        
-        This method is used to recursively convert the object into a
-        self-documenting form that can easily be used for various
-        purposes, including diffs and queries.
-
-        These structures may be primitive Python types (string,
-        integer, boolean, etc.) or complex Python types (lists,
-        tuples, or dicts). If the return type is a dict, then the keys
-        of the dict match the fieldname of the object. If the return
-        struct (or value of a dict key) is a list, then it is a list
-        of structs. Otherwise, the struct is just the value of the
-        attribute.
-
-        :returns: Returns a struct containing the data of the object.
-        :rtype: dict
-        """
-        return {"_class": "Attribute",
-                "private": PrivacyBase.serialize(self),
-                "citation_list": CitationBase.to_struct(self),
-                "note_list": NoteBase.to_struct(self),
-                "type": self.type.to_struct(), 
-                "value": self.value}
-
-    @classmethod
-    def from_struct(cls, struct):
-        """
-        Given a struct data representation, return a serialized object.
-
-        :returns: Returns a serialized object
-        """
-        return (PrivacyBase.from_struct(struct["private"]),
-                CitationBase.from_struct(struct["citation_list"]),
-                NoteBase.from_struct(struct["note_list"]),
-                AttributeType.from_struct(struct["type"]), 
-                struct["value"])
+        return (
+            PrivacyBase.serialize(self),
+            CitationBase.serialize(self),
+            NoteBase.serialize(self),
+            self.type.serialize(),
+            self.value,
+        )
 
     def unserialize(self, data):
         """
@@ -297,16 +228,46 @@ class Attribute(AttributeRoot, CitationBase, NoteBase):
         self.type.unserialize(the_type)
         return self
 
+    @classmethod
+    def get_schema(cls):
+        """
+        Returns the JSON Schema for this class.
+
+        :returns: Returns a dict containing the schema.
+        :rtype: dict
+        """
+        return {
+            "type": "object",
+            "title": _("Attribute"),
+            "properties": {
+                "_class": {"enum": [cls.__name__]},
+                "private": {"type": "boolean", "title": _("Private")},
+                "citation_list": {
+                    "type": "array",
+                    "title": _("Citations"),
+                    "items": {"type": "string", "maxLength": 50},
+                },
+                "note_list": {
+                    "type": "array",
+                    "items": {"type": "string", "maxLength": 50},
+                    "title": _("Notes"),
+                },
+                "type": AttributeType.get_schema(),
+                "value": {"type": "string", "title": _("Value")},
+            },
+        }
+
     def get_referenced_handles(self):
         """
         Return the list of (classname, handle) tuples for all directly
         referenced primary objects.
-        
+
         :returns: List of (classname, handle) tuples for referenced objects.
         :rtype: list
         """
-        return self.get_referenced_note_handles() + \
-                self.get_referenced_citation_handles()
+        return (
+            self.get_referenced_note_handles() + self.get_referenced_citation_handles()
+        )
 
     def merge(self, acquisition):
         """
